int dummy=0; // this is crap to force sketch to put arduino include here

#define MODE_UNKNOWN 0
#define MODE_PoV     1
#define MODE_RANDOM  2
#define MODE_CYLON   3

#define MAX_COLS 96
#define SERIAL_BUF_LEN (MAX_COLS+15)
#include <ioUtil.h>
SerialLineBuffer LineBuf;

struct {
  short nCols;  // no. columns in buffer
  short spaceCols; // no. columns "space" time before repeat or reverse
  short mode;  // MODE_ code from above
  short cylonCols;  // no. cols of time for each cylon flash
  int colTime;  // milliseconds/column
  int misc[3];  // reserved for future use
  short disp[MAX_COLS];  // display flags
} State;

// These are the DI/O pins used for the display
#define NPINS 10
int ledPins[NPINS] = {12,11,10,9,8,7,6,5,4,3};

#include <EEPROM.h>

void loadState()
{
  int n = sizeof(State);
  byte *bp = (byte *)(&State);
  for(int i=0; i < n; i++, bp++) *bp = EEPROM.read(i);

  if (!validState()) initState();
}
void saveState()
{
  int n = sizeof(State);
  byte *bp = (byte *)(&State);
  for(int i=0; i < n; i++, bp++) EEPROM.write(i,*bp);
}

// set state to a reasonable default
void initState()
{
  State.nCols = 2;
  State.spaceCols = 1;
  State.cylonCols = 10;
  State.mode = MODE_PoV;
  State.colTime = 10; // ms
  for (int i=0; i < MAX_COLS; i++) State.disp[i] = (i&1)?0x5555:0x2aaa;
  saveState();
}

void setup()
{
  int i;
  for (i=0;i<NPINS;i++)
  {
    pinMode(ledPins[i],OUTPUT);
  }
  pinMode(13,OUTPUT);  // use on-board LED

  // restore state from EEPROM
  loadState();
  Serial.begin(9600);
}


void loop()
{
  checkCommand();
  int i,dt,k;
  dt = State.colTime;
  switch (State.mode)
    {
    case MODE_CYLON:
      dt *= State.cylonCols;
      for (i=0; i < NPINS; i++)
        {
          digitalWrite(ledPins[i],HIGH);
          delay(dt);
          digitalWrite(ledPins[(i+NPINS-1)%NPINS],LOW);
          delay(dt);
        }
      for (i=NPINS-2; i >= 0; i--)
        {
          digitalWrite(ledPins[i],HIGH);
          delay(dt);
          digitalWrite(ledPins[(i+1)%NPINS],LOW);
          delay(dt);
        }
      break;
    case MODE_PoV:
      for (i=0; i < State.nCols; i++)
        {
          short mask=1;
          for (k=0; k < NPINS; k++, mask <<= 1)
            digitalWrite(ledPins[k],(mask & State.disp[i])?HIGH:LOW);
        }
      for (k=0; k < NPINS; k++) digitalWrite(ledPins[k],LOW);
      delay(State.spaceCols*dt);
      break;
    default:  // random default
      {
        dt *= 10;
        k = random(100);
        int lvl = (k<50)?LOW:HIGH;
        int j = random(NPINS);
        digitalWrite(ledPins[j],lvl);
        delay(dt);
      }
  }
  digitalWrite(13,digitalRead(13)?LOW:HIGH);  // toggle heartbeat
}

// poll for commands from serial port
void checkCommand()
{
  short mask;
  if (!LineBuf.isComplete()) return;
  char key = lowCase(*(LineBuf.get()));
  switch(key)
    {
      //short mask;
      //int k;
      //char *b;
    case 'h' :
      Serial.println("  h)  help (print this message)");
      Serial.println("  s)  save state");
      Serial.println("  r)  random lights mode");
      Serial.println("  c)  cylon mode");
      Serial.println("  p)  PoV sign mode");
      Serial.println("  n)  no. cols to display");
      Serial.println("  t)  col time, ms");
      Serial.println("  b)  blank cols between repeat");
      Serial.println("  i)  re-Initialize state");
      Serial.print(  " Lx)  Set pattern for line x, 0<=x<=");
      Serial.println(NPINS);
      break;
    case 's' : saveState(); break;
    case 'r' : State.mode = MODE_RANDOM; break;
    case 'p' : State.mode = MODE_PoV; break;
    case 'c' : State.mode = MODE_CYLON; break;
    case 'i' : initState(); break;
    case 'n' : State.nCols    =nextInt(LineBuf.get()+1); break;
    case 't' : State.colTime  =nextInt(LineBuf.get()+1); break;
    case 'b' : State.spaceCols=nextInt(LineBuf.get()+1); break;
    case 'l' :
    {
      char *b = LineBuf.get()+1;
      int k = ((int)(*b)) - ((int)'0');
      if ((k<0) || (k > 15)) break;
      b++;
      short mask = (short)(1<<k);
      for (int i=0; i < State.nCols; i++, b++)
        {
          if (isOn(*b)) State.disp[i] |=  mask;
          else          State.disp[i] &= ~mask;
        }
      break;
    }
    default :
      Serial.print("Unrecognized Command : <");
      Serial.print(LineBuf.get());
      Serial.println(">");
      Serial.println("Send command h for help.");
    }
    printState();
    printMsg();
}

void printState()
{
  Serial.print(State.nCols);
  Serial.print(" Columns   ");
  Serial.print(State.spaceCols);
  Serial.print("   ");
  Serial.print(State.colTime);
  Serial.println("ms/col");
  Serial.flush();
}
void printMsg()
{
  int i,k;
  Serial.println();
  for (i=0; i < State.nCols; i++) Serial.print("-");
  short mask=1;
  for(k=0; k < NPINS; k++, mask <= 1)
    {
      for (i=0; i < State.nCols; i++) Serial.print(State.disp[k]&mask?"X":" ");
      Serial.println("|");
    }
  Serial.println();
  for (i=0; i < State.nCols; i++) Serial.print("-");
  Serial.println();
  Serial.flush();
}
// parse next int from a string
int nextInt(const char *s)
{
  const char *c = s;
  int val = 0;
  for(;;)
    {
      int k = ((int)(*c)) - ((int)'0');
      if ((k<0)||(k>9)) return val;
      val *= 10;
      val += k;
      c++;
    }
}

bool isOn(const char c)
{
  if ((c=='0') || (c=='.') || (c==' ') || (c==0)) return false;
  //if ((c=='1')||(lowCase(c)=='x')) return true;
  return true;
}

bool validState()
{
  // check for silly state, set to default if inconsistent
  if ((State.mode <= 0) || (State.mode >  3) ||
      (State.nCols<  1) || (State.nCols>MAX_COLS)) return false;
  if ((State.spaceCols < 1) || (State.spaceCols > 10*MAX_COLS)) return false;
  if ((State.colTime < 1)   || (State.colTime > 10000)) return false;
  if ((State.cylonCols < 1) || (State.cylonCols > 10000)) return false;
  return true;
}

